

intrinsic TestBlowup(max::RngIntElt: statisticsFile := "")
{ test procedure for parametrizing a blowup of a projective plane }

  //  ##  begin write output
  outputFile := "blowup-examples";
  //  ##  end write outup

  //  Par<s,t,u> := PolynomialRing(RationalField(), 3);

  R<z1,z2,z3,z4,z5,z6,z7,z8,z9> := PolynomialRing(Rationals(), 9);

  //  f := hom<R -> Par | s^2*t, s^2*u, 
  //                      s*t^2, s*t*u, s*u^2,
  //                      t^3, t^2*u, t*u^2, u^3>;
  //  I := Implicitization(f);


  repeat
    //  ###   generate an example

    print "--------------";
    print "example started";
    print "output written into 'blowup-examples'";

    b := true;
    while b do
      M := ZeroMatrix(Rationals(), 9,9);
      for i := 1 to 9 do
        for j := 1 to 9 do
          M[i][j] := Random(-max, max);
        end for;
      end for;
      b := IsSingular(M);
    end while;

    c := [R|];
    for i := 1 to 9 do
      c[i] := M[i][1]*z1 + M[i][2]*z2 + M[i][3]*z3 + 
              M[i][4]*z4 + M[i][5]*z5 + M[i][6]*z6 +
              M[i][7]*z7 + M[i][8]*z8 + M[i][9]*z9;
    end for;

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Perturbation matrix:");
      Write(statisticsFile, M);
    end if;
    //  #######  end write statistics  #######

    I := ideal<R | [
    -c[1]*c[4] + c[2]*c[3],
    -c[1]*c[6] + c[3]^2,
    -c[1]*c[5] + c[2]*c[4],
    -c[1]*c[7] + c[3]*c[4],
    -c[1]*c[8] + c[4]^2,
    -c[1]*c[8] + c[3]*c[5],
    -c[1]*c[9] + c[4]*c[5],
    -c[2]*c[9] + c[5]^2,
    -c[1]*c[7] + c[2]*c[6],
    -c[3]*c[7] + c[4]*c[6],
    -c[3]*c[8] + c[5]*c[6],
    -c[1]*c[8] + c[2]*c[7],
    -c[3]*c[8] + c[4]*c[7],
    -c[3]*c[9] + c[5]*c[7],
    -c[6]*c[8] + c[7]^2,
    -c[1]*c[9] + c[2]*c[8],
    -c[3]*c[9] + c[4]*c[8],
    -c[4]*c[9] + c[5]*c[8],
    -c[6]*c[9] + c[7]*c[8],
    -c[7]*c[9] + c[8]^2]>;

    //  ##  begin write output
    Write(outputFile, "Parametrizing the surface:\n   ");
    Write(outputFile, I);
    //  ##  end write output

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      cf_max := 0;
      for i := 1 to #Basis(I) do
        cf := Coefficients(Basis(I)[i]);
        for j := 1 to #cf do
          cf[j] := Abs(cf[j]);
        end for;
        cf_max := Max(cf_max, Max(cf));
      end for;
      Write(statisticsFile, "Maximal abs value of coef in implicit eqns:");
      Write(statisticsFile, cf_max);
    end if;
    //  ######  end write statistics  ######

    //  parametrize it
    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      t_all := Cputime();
    end if;
    //  ######  end write statistics  ######

    prm := ParametrizeBlowup(I: statisticsFile := statisticsFile);

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Overall parametrization time:");
      Write(statisticsFile, Cputime(t_all));
    end if;
    //  ######  end write statistics  ######

    //  ##  begin debug info
    Write(outputFile, "\nParametrization:");
    Write(outputFile, prm);
    Write(outputFile, "\n-----------------------------\n");
    //  ##  end debug info

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      cf_max := 0;
      for i := 1 to #prm do
        cf := Coefficients(prm[i]);
        for j := 1 to #cf do
          cf[j] := Abs(cf[j]);
        end for;
        for j := 1 to #cf do
          cf_max := Max([cf_max, 1+Floor(Log(10, Numerator(cf[j]))), 
                                 1+Floor(Log(10, Denominator(cf[j])))]);
        end for;
      end for;
      Write(statisticsFile, 
        "Maximal number of digits in num/denom of consts in parametrization:");
      Write(statisticsFile, cf_max);
      Write(statisticsFile, "###############################################");
    end if;
    //  ######  end write statistics  ######


    //  prm is the parametrization, check it:
    phi := hom<R -> Parent(prm[1]) | prm[1], prm[2], prm[3], prm[4], prm[5],
                                     prm[6], prm[7], prm[8], prm[9]>;
    ok := true;
    for i := 1 to #Basis(I) do
      if (phi(Basis(I)[i]) ne 0) then
        ok := false;
        break i;
      end if;
    end for;
    if ok then print "ok";
    else print "wrong!";
    end if;

    print "example ended";
    print "(press 'q<Enter>' to quit, just <Enter> for another example)";

    read hocico;
  until (hocico eq "q");

end intrinsic;


//TestBlowup(15);



intrinsic TestAnticanonicalP1xP1(max::RngIntElt: Diag := false, statisticsFile := "")
{ test procedure for parametrizing an anticanonically embedded P1xP1 }

  //  ##  begin write output
  outputFile := "anticanonicalP1xP1-examples";
  //  ##  end write outup

  R<z1,z2,z3,z4,z5,z6,z7,z8,z9> := PolynomialRing(Rationals(), 9);

  repeat
    //  ###   generate an example
    print "--------------";
    print "example started";
    print "output written into 'anticanonicalP1xP1-examples'";

    b := true;
    while b do
      M := ZeroMatrix(Rationals(), 9,9);
      for i := 1 to 9 do
        for j := 1 to 9 do
          M[i][j] := Random(-max, max);
        end for;
      end for;
      b := IsSingular(M);
    end while;

    c := [R|];
    for i := 1 to 9 do
      c[i] := M[i][1]*z1 + M[i][2]*z2 + M[i][3]*z3 + 
              M[i][4]*z4 + M[i][5]*z5 + M[i][6]*z6 +
              M[i][7]*z7 + M[i][8]*z8 + M[i][9]*z9;
    end for;

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Perturbation matrix:");
      Write(statisticsFile, M);
    end if;
    //  #######  end write statistics  #######

    I := ideal<R | [
    c[1]*c[3] - c[2]^2,
    c[1]*c[5] - c[2]*c[4],
    c[1]*c[6] - c[3]*c[4],
    c[1]*c[7] - c[4]^2,
    c[1]*c[6] - c[2]*c[5],
    c[2]*c[6] - c[3]*c[5],
    c[1]*c[8] - c[4]*c[5],
    c[1]*c[9] - c[5]^2,
    c[1]*c[9] - c[4]*c[6],
    c[2]*c[9] - c[5]*c[6],
    c[3]*c[9] - c[6]^2,
    c[1]*c[8] - c[2]*c[7],
    c[1]*c[9] - c[3]*c[7],
    c[4]*c[8] - c[5]*c[7],
    c[4]*c[9] - c[6]*c[7],
    c[1]*c[9] - c[2]*c[8],
    c[2]*c[9] - c[3]*c[8],
    c[4]*c[9] - c[5]*c[8],
    c[5]*c[9] - c[6]*c[8],
    c[7]*c[9] - c[8]^2]>;

    //  ##  begin write outpup
    Write(outputFile, "Parametrizing the surface:\n   ");
    Write(outputFile, I);
    //  ##  end write output

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      cf_max := 0;
      for i := 1 to #Basis(I) do
        cf := Coefficients(Basis(I)[i]);
        for j := 1 to #cf do
          cf[j] := Abs(cf[j]);
        end for;
        cf_max := Max(cf_max, Max(cf));
      end for;
      Write(statisticsFile, "Maximal abs value of coef in implicit eqns:");
      Write(statisticsFile, cf_max);
    end if;
    //  ######  end write statistics  ######

    //  parametrize it
    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      t_all2 := Cputime();
    end if;
    //  ######  end write statistics  ######

    prm := ParametrizeAnticanonicalP1xP1(
      I: Diag := Diag, statisticsFile := statisticsFile);

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Overall parametrization time:");
      Write(statisticsFile, Cputime(t_all2));
    end if;
    //  ######  end write statistics  ######

    //  ##  begin write output
    Write(outputFile, "\nParametrization:");
    Write(outputFile, prm);
    Write(outputFile, "\n-----------------------------\n");
    //  ##  end write output

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      cf_max := 0;
      for i := 1 to #prm do
        cf := Coefficients(prm[i]);
        for j := 1 to #cf do
          cf[j] := Abs(cf[j]);
        end for;
        for j := 1 to #cf do
          cf_max := Max([cf_max, 1+Floor(Log(10, Numerator(cf[j]))), 
                                 1+Floor(Log(10, Denominator(cf[j])))]);
        end for;
      end for;
      Write(statisticsFile, 
        "Maximal number of digits in num/denom of consts in parametrization:");
      Write(statisticsFile, cf_max);
      Write(statisticsFile, "###############################################");
    end if;
    //  ######  end write statistics  ######


    //  prm is the parametrization, check it:
    phi := hom<R -> Parent(prm[1]) | prm[1], prm[2], prm[3], prm[4], prm[5],
                                     prm[6], prm[7], prm[8], prm[9]>;
    ok := true;
    for i := 1 to #Basis(I) do
      if (phi(Basis(I)[i]) ne 0) then
        ok := false;
        break i;
      end if;
    end for;
    if ok then print "ok";
    else print "wrong!";
    end if;

    print "example ended";
    print "(press 'q<Enter>' to quit, just <Enter> for another example)";

    read hocico;
  until (hocico eq "q");

end intrinsic;


//TestAnticanonicalP1xP1(2);





//  #######################################################
//  ##  produce more Gauss-like random numbers
//  #######################################################
function Rnd(min, max)

  d := 2;

  sum := 0;
  for i := 1 to d do
    sum +:= Random(min, max);
  end for;

  return Truncate(sum/d);

end function;


//  #######################################################
//  ##  maxf: maximum for the field extension
//  ##  mafc: maximum for the perturbation of coeffs
//  #######################################################
intrinsic TestAnticanonicalSphere(max::RngIntElt, maxf::RngIntElt: statisticsFile := "")
{ test procedure for parametrizing an anticanonically embedded sphere }

  //  ##  begin write output
  outputFile := "anticanonicalSphere-examples";
  //  ##  end write outup

  RR<s,t,u, y1,y2,y3,y4,y5,y6,y7,y8,y9> := PolynomialRing(Rationals(), 12);
  R<z1,z2,z3,z4,z5,z6,z7,z8,z9> := PolynomialRing(Rationals(), 9);

  proj := hom<RR -> R | 0,0,0, z1,z2,z3,z4,z5,z6,z7,z8,z9>;

  // //  this would be a parametrization of x1^2 + x2^2 + x3^2 = x0^2
  // x0 := u^2 + s^2 + t^2;
  // x1 := 2*s*u;
  // x2 := (-2*t*u);
  // x3 := u^2 - s^2 - t^2;

  repeat
    //  ###   generate an example
    print "--------------";
    print "example started";
    print "output written into 'anticanonicalSphere-examples'";

    //  choose the splitting field
    repeat
      k := Random(-maxf, maxf);
    until (not IsSquare(k));

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Discriminant of the field:");
      Write(statisticsFile, k);
    end if;
    //  ######  end write statistics  ######

    //  parametrization of x0^2 - x1^2 = x2^2 - k*x3^2
    x0 := s^2 - k*t^2 + u^2;
    x1 := -s^2 + k*t^2 + u^2;
    x2 := 2*s*u;
    x3 := -2*t*u;

    //  lineal system of the canonical divisor consists of all
    //  quadratic forms in x0,x1,x2,x3
    //  but x0 = x1^2 + x2^2 - k*x3^2, therefore
    //  this one in not needed in the basis

    I := ideal< RR | [ 
       y1 - x0*x1,
       y2 - x0*x2,
       y3 - x0*x3,
       y4 - x1^2,
       y5 - x1*x2,
       y6 - x1*x3,
       y7 - x2^2,
       y8 - x2*x3,
       y9 - x3^2
    ]>;

    eI := EliminationIdeal(I, 3);

    b := [R | ];
    for i := 1 to #Basis(eI) do
      Append(~b, proj(Basis(eI)[i]));
    end for;
    peI := ideal< R | b >;

    //  perturbe the coefficients
    b := true;
    while b do
      M := ZeroMatrix(Rationals(), 9,9);
      for i := 1 to 9 do
        for j := 1 to 9 do
          M[i][j] := Rnd(-max, max);
          //M[i][j] := Random(-max, max);
        end for;
      end for;
      b := IsSingular(M);
    end while;

    c := [R|];
    for i := 1 to 9 do
      c[i] := M[i][1]*z1 + M[i][2]*z2 + M[i][3]*z3 + 
              M[i][4]*z4 + M[i][5]*z5 + M[i][6]*z6 +
              M[i][7]*z7 + M[i][8]*z8 + M[i][9]*z9;
    end for;

    //  #######  begin write statistics  #######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Perturbation matrix:");
      Write(statisticsFile, M);
    end if;
    //  #######  end write statistics  #######

    perPhi := hom<R -> R | c[1],c[2],c[3],c[4],c[5],c[6],c[7],c[8],c[9]>;
    perPhiB := [R | ];
    for i := 1 to #Basis(peI) do
      Append(~perPhiB, perPhi(Basis(peI)[i]));
    end for;
    I := ideal< R | perPhiB >;

    //  #######  begin write statistics  #######
    if not (statisticsFile eq "") then
      cf_max := 0;
      for i := 1 to #Basis(I) do
        cf := Coefficients(Basis(I)[i]);
        for j := 1 to #cf do
          cf[j] := Abs(cf[j]);
        end for;
        cf_max := Max(cf_max, Max(cf));
      end for;
      Write(statisticsFile, "Maximal abs value of coef in implicit eqns:");
      Write(statisticsFile, cf_max);
    end if;
    //  #######  end write statistics  #######

    //  ##  begin write outpup
    Write(outputFile, "Parametrizing the surface:\n   ");
    Write(outputFile, I);
    //  ##  end write output


    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      t_all := Cputime();
    end if;
    //  ######  end write statistics  ######

    //
    //  print "Parametrizing over extension...";
    //  
    //  prm1 := ParametrizeAnticanonicalOverExtension(I);
    //  
    //  //  ##  begin write output
    //  Write(outputFile, 
    //    "Intermediate parametrization");
    //  Write(outputFile, "over the splitting field: ");
    //  Write(outputFile, CoefficientRing(Parent(prm1[1])));
    //  Write(outputFile, prm1);
    //  //  Write(outputFile, "\n............................\n");
    //  //  ##  end write output
    //  
    //  print "Intersecting conjugates...";
    //  
    //  prm2 := AnticanonicalIntersectConjugates(prm1);

    prm2 := ParametrizeAnticanonicalSphere(
      I: statisticsFile := statisticsFile);

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      Write(statisticsFile, "Overall parametrization time:");
      Write(statisticsFile, Cputime(t_all));
    end if;
    //  ######  end write statistics  ######

    //  ##  begin write output
    Write(outputFile, "\nFinal parametrization:");
    Write(outputFile, prm2);
    Write(outputFile, "\n-----------------------------\n");
    //  ##  end write output

    //  ######  begin write statistics  ######
    if not (statisticsFile eq "") then
      cf_max := 0;
      for i := 1 to #prm2 do
        cf := Coefficients(prm2[i]);
        for j := 1 to #cf do
          cf[j] := Abs(cf[j]);
        end for;
        for j := 1 to #cf do
          cf_max := Max([cf_max, 1+Floor(Log(10, Numerator(cf[j]))), 
                                 1+Floor(Log(10, Denominator(cf[j])))]);
        end for;
      end for;
      Write(statisticsFile, 
        "Maximal number of digits in num/denom of consts in parametrization:");
      Write(statisticsFile, cf_max);
      Write(statisticsFile, "###############################################");
    end if;
    //  ######  end write statistics  ######

    //  prm2 is the parametrization, check it:
    phi := hom<R -> Parent(prm2[1]) |
      prm2[1], prm2[2], prm2[3], prm2[4], prm2[5], prm2[6], prm2[7], prm2[8], prm2[9]>;

    ok := true;
    for i := 1 to #Basis(I) do
      if (phi(Basis(I)[i]) ne 0) then
        ok := false;
        break i;
      end if;
    end for;
    if ok then print "ok";
    else print "wrong!";
    end if;

    print "example ended";
    print "(press 'q<Enter>' to quit, just <Enter> for another example)";

    read hocico;
  until (hocico eq "q");

end intrinsic;


//TestAnticanonicalSphere(3,1);

